@@ -589,6 +589,18 @@ def get_neuropixel_version_from_files(ephys_files):
         return '3A'
 
 
+def get_probes_from_folder(session_path):
+    # should glob the ephys files and get out the labels
+    # This assumes the meta files exist on the server (this is the case for now but should it be?)
+    ephys_files = glob_ephys_files(session_path, ext='meta')
+    probes = []
+    for files in ephys_files:
+        if files['label']:
+            probes.append(files['label'])
+
+    return probes
+
+
 def glob_ephys_files(session_path, suffix='.meta', ext='bin', recursive=True, bin_exists=True):
     """
     From an arbitrary folder (usually session folder) gets the ap and lf files and labels
