@@ -5,6 +5,8 @@ from inspect import getmembers, ismethod
 
 import numpy as np
 import copy
+import random
+import string
 
 from one.api import ONE
 from ibllib.tests import TEST_DB
@@ -23,7 +25,8 @@ brain_atlas = AllenAtlas(25)
 class TestProbeInsertion(unittest.TestCase):
 
     def test_creation(self):
-        probe = ['probe00', 'probe01']
+        probe = [''.join(random.choices(string.ascii_letters, k=5)),
+                 ''.join(random.choices(string.ascii_letters, k=5))]
         create_alyx_probe_insertions(session_path=EPHYS_SESSION, model='3B2', labels=probe,
                                      one=one, force=True)
         insertion = one.alyx.get(f'/insertions?&session={EPHYS_SESSION}', clobber=True)
@@ -41,7 +44,8 @@ class TestTracingQc(unittest.TestCase):
 
     @classmethod
     def setUpClass(cls) -> None:
-        probe = ['probe00', 'probe01']
+        probe = [''.join(random.choices(string.ascii_letters, k=5)),
+                 ''.join(random.choices(string.ascii_letters, k=5))]
         ins = create_alyx_probe_insertions(session_path=EPHYS_SESSION, model='3B2', labels=probe,
                                            one=one, force=True)
         cls.probe00_id, cls.probe01_id = (x['id'] for x in ins)
@@ -87,6 +91,7 @@ class TestAlignmentQcExisting(unittest.TestCase):
             list(np.array(cls.alignments['2020-06-26T16:40:14_Karolina_Socha'][1]) + 0.0001)
         cls.cluster_chns = data['cluster_chns']
         insertion = data['insertion'].tolist()
+        insertion['name'] = ''.join(random.choices(string.ascii_letters, k=5))
         insertion['json'] = {'xyz_picks': cls.xyz_picks}
         probe_insertion = one.alyx.rest('insertions', 'create', data=insertion)
         cls.probe_id = probe_insertion['id']
@@ -198,6 +203,7 @@ class TestAlignmentQcManual(unittest.TestCase):
                             joinpath('fixtures', 'qc', 'data_alignmentqc_existing.npz')),
                        allow_pickle=True)
         insertion = data['insertion'].tolist()
+        insertion['name'] = ''.join(random.choices(string.ascii_letters, k=5))
         insertion['json'] = {'xyz_picks': cls.xyz_picks}
         probe_insertion = one.alyx.rest('insertions', 'create', data=insertion)
         cls.probe_id = probe_insertion['id']
